﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media
Imports System.Windows.Navigation
Imports System.Windows.Shapes
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell

Partial Public Class MainPage
    Inherits PhoneApplicationPage
    Private numRowsCols As Integer = 4
    Private captureSource As CaptureSource
    Private videoBrush As VideoBrush

    Public Sub New()
        InitializeComponent()
        fewerButton = TryCast(Me.ApplicationBar.Buttons(0), ApplicationBarIconButton)
        moreButton = TryCast(Me.ApplicationBar.Buttons(1), ApplicationBarIconButton)

        captureSource = New CaptureSource With {.VideoCaptureDevice = CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice()}
        videoBrush = New VideoBrush()
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
        If Me.State.ContainsKey("numRowsCols") Then
            numRowsCols = CInt(Fix(Me.State("numRowsCols")))
            Me.State.Remove("numRowsCols")
        End If
        CreateRowsAndColumns()

        If captureSource IsNot Nothing AndAlso CaptureDeviceConfiguration.AllowedDeviceAccess Then
            videoBrush.SetSource(captureSource)
            captureSource.Start()
            startButton.Visibility = Visibility.Collapsed
        End If

        MyBase.OnNavigatedTo(args)
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
        Me.State.Add("numRowsCols", numRowsCols)

        If captureSource IsNot Nothing AndAlso captureSource.State = CaptureState.Started Then
            captureSource.Stop()
            startButton.Visibility = Visibility.Visible
        End If

        MyBase.OnNavigatedFrom(args)
    End Sub

    Private Sub OnStartButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        If captureSource IsNot Nothing AndAlso (CaptureDeviceConfiguration.AllowedDeviceAccess OrElse CaptureDeviceConfiguration.RequestDeviceAccess()) Then
            videoBrush.SetSource(captureSource)
            captureSource.Start()
            startButton.Visibility = Visibility.Collapsed
        End If
    End Sub

    Private Sub OnFewerButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        numRowsCols -= 1
        CreateRowsAndColumns()
    End Sub

    Private Sub OnMoreButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        numRowsCols += 1
        CreateRowsAndColumns()
    End Sub

    Private Sub CreateRowsAndColumns()
        videoPanel.Children.Clear()
        videoPanel.RowDefinitions.Clear()
        videoPanel.ColumnDefinitions.Clear()

        For row As Integer = 0 To numRowsCols - 1
            videoPanel.RowDefinitions.Add(New RowDefinition With {.Height = New GridLength(1, GridUnitType.Star)})
        Next row

        For col As Integer = 0 To numRowsCols - 1
            videoPanel.ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(1, GridUnitType.Star)})
        Next col

        For row As Integer = 0 To numRowsCols - 1
            For col As Integer = 0 To numRowsCols - 1
                Dim rect As Rectangle = New Rectangle With {.Fill = videoBrush, .RenderTransformOrigin = New Point(0.5, 0.5), .RenderTransform = New ScaleTransform With {.ScaleX = 1 - 2 * (col Mod 2), .ScaleY = 1 - 2 * (row Mod 2)}}
                Grid.SetRow(rect, row)
                Grid.SetColumn(rect, col)
                videoPanel.Children.Add(rect)
            Next col
        Next row
        fewerButton.IsEnabled = numRowsCols > 1
    End Sub
End Class