﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Media
Imports System.Windows.Media.Imaging

Public Class KaleidoscopeVideoSink
    Inherits VideoSink
    Private bitmap As WriteableBitmap
    Private videoFormat As VideoFormat
    Private indexMap() As Integer ' maps bitmap pixel index to video pixel index

    Public Sub New(ByVal bitmap As WriteableBitmap)
        Me.bitmap = bitmap
        indexMap = New Integer(bitmap.PixelWidth * bitmap.PixelHeight - 1) {}
    End Sub

    Protected Overrides Sub OnCaptureStarted()
    End Sub

    Protected Overrides Sub OnCaptureStopped()
    End Sub

    Protected Overrides Sub OnFormatChange(ByVal videoFormat As VideoFormat)
        Me.videoFormat = videoFormat

        ' Define a source equilateral triangle of the video 
        Dim side As Integer = CInt(Fix(videoFormat.PixelHeight / Math.Sqrt(0.75)))
        Dim xCenter As Integer = videoFormat.PixelWidth / 2
        Dim A As New Point(xCenter - side \ 2, 0) ' top left vertex
        Dim B As New Point(xCenter + side \ 2, 0) ' top right vertex
        Dim C As New Point(xCenter, videoFormat.PixelHeight - 1) ' bottom vertex

        ' Go down the row and call GenerateIndexMap for each triangle
        GenerateIndexMap(0, 0, 104, A, B, C)
        GenerateIndexMap(104, 60, 0, C, A, B)
        GenerateIndexMap(0, 120, 104, B, C, A)
        GenerateIndexMap(104, 180, 0, A, B, C)
        GenerateIndexMap(0, 240, 104, C, A, B)
        GenerateIndexMap(104, 300, 0, B, C, A)
        GenerateIndexMap(0, 360, 104, A, B, C)
        GenerateIndexMap(104, 420, 0, C, A, B)
        GenerateIndexMap(0, 480, 104, B, C, A)
    End Sub

    ' First three arguments reference the WritableBitmap destination;
    '  Second three arguments reference a triangle in the video source
    Private Sub GenerateIndexMap(ByVal xApex As Integer, ByVal yApex As Integer, ByVal xBase As Integer, ByVal ptApex As Point, ByVal ptTopBase As Point, ByVal ptBotBase As Point)
        ' Derive information about source triangle
        Dim ptBaseCenter As New Point((ptTopBase.X + ptBotBase.X) / 2, (ptTopBase.Y + ptBotBase.Y) / 2)
        Dim vector As New Point(ptBotBase.X - ptTopBase.X, ptBotBase.Y - ptTopBase.Y)

        ' From apex of destination triangle to base
        For x As Integer = 0 To 104
            Dim xDst As Integer = ((104 - x) * xApex + x * xBase) / 104
            Dim y1 As Integer = -60 * x \ 104
            Dim y2 As Integer = 60 * x \ 104
            Dim xRel As Double = x / 104.0

            ' Across the triangle
            For y As Integer = y1 To y2
                Dim yDst As Integer = yApex + y
                Dim yRel As Double = y / 120.0

                If yDst >= 0 AndAlso yDst < bitmap.PixelHeight Then
                    Dim ptAltitude As New Point((1 - xRel) * ptApex.X + xRel * ptBaseCenter.X, (1 - xRel) * ptApex.Y + xRel * ptBaseCenter.Y)
                    Dim ptSource As New Point(ptAltitude.X + yRel * vector.X, ptAltitude.Y + yRel * vector.Y)
                    Dim xSrc As Integer = CInt(Fix(ptSource.X))
                    Dim ySrc As Integer = CInt(Fix(ptSource.Y))
                    Dim indexSrc As Integer = ySrc * videoFormat.PixelWidth + xSrc
                    Dim indexDst As Integer = bitmap.PixelWidth * yDst + xDst
                    indexMap(indexDst) = indexSrc
                End If
            Next y
        Next x
    End Sub

    Protected Overrides Sub OnSample(ByVal sampleTimeInHundredNanoseconds As Long, ByVal frameDurationInHundredNanoseconds As Long, ByVal sampleData() As Byte)
        For bitmapIndex As Integer = 0 To indexMap.Length - 1
            ' Assumes 32-bit color
            Dim videoIndex As Integer = 4 * indexMap(bitmapIndex)

            Dim pixel As Integer = (CInt(sampleData(videoIndex + 0)) << 0) Or (CInt(sampleData(videoIndex + 1)) << 8) Or (CInt(sampleData(videoIndex + 2)) << 16) Or (CInt(sampleData(videoIndex + 2)) << 24)

            bitmap.Pixels(bitmapIndex) = pixel
        Next bitmapIndex

        bitmap.Dispatcher.BeginInvoke(Sub() bitmap.Invalidate())
    End Sub

End Class